%scan.m
clear all;
global fit1 fit2 fit3 fit4 convert
scan_data = csvread('DAC_SCAN.txt');
scan_size = size(scan_data);
LUT = fopen('DC_LUT.txt','w');
ADC = fopen('ADC_conversion.txt','w');

dac_hi = 3400;
dac_lo = 10;
dac_step = 20;
convert = 1.2/4096;

y=transpose(scan_data);
     y1 = y(1,:);
     y2 = y(2,:);
     y3 = y(3,:);
     y4 = y(4,:);
x = dac_lo:dac_step:dac_hi;
x_voltage = convert*x;

fit_hi=150;
fit_lo=100;

fit1=polyfit(x(fit_lo:fit_hi),y1(fit_lo:fit_hi),1);
slope1 = sprintf('%6.4f',fit1(1)/(convert*1000));

fit2=polyfit(x(fit_lo:fit_hi),y2(fit_lo:fit_hi),1);
slope2 = sprintf('%6.4f',fit2(1)/(convert*1000));

fit3=polyfit(x(fit_lo:fit_hi),y3(fit_lo:fit_hi),1);
slope3 = sprintf('%6.4f',fit3(1)/(convert*1000));

fit4=polyfit(x(fit_lo:fit_hi),y4(fit_lo:fit_hi),1);
slope4 = sprintf('%6.4f',fit4(1)/(convert*1000));

fprintf('Counts/mV conversions:\nCH1 : %s\nCH2 : %s\nCH3 : %s\nCH4 : %s\n',slope1,slope2,slope3,slope4);
fprintf(ADC, '%s\n', slope1);
fprintf(ADC, '%s\n', slope2);
fprintf(ADC, '%s\n', slope3);
fprintf(ADC, '%s\n', slope4);
fclose(ADC);

%%calculate residuals
res1 = y1-(fit1(1)*x+fit1(2));
res2 = y2-(fit2(1)*x+fit2(2));
res3 = y3-(fit3(1)*x+fit3(2));
res4 = y4-(fit4(1)*x+fit4(2));

%%spline interpolation to get full LUT
xx1 =  y1(1):1:y1(length(y1));
res_spline_1=zeros(1,2048);
res_spline_1a=spline(y1,res1,xx1);
     n=size(res_spline_1a);
     for  i=1:n(2),
          res_spline_1(i) = res_spline_1a(i);
     end

xx2 =  y2(1):1:y2(length(y2));
res_spline_2=zeros(1,2048);
res_spline_2a=spline(y2,res2,xx2);
     n=size(res_spline_2a);
     for  i=1:n(2),
          res_spline_2(i) = res_spline_2a(i);
     end

xx3 =  y3(1):1:y3(length(y3));
res_spline_3=zeros(1,2048);
res_spline_3a=spline(y3,res3,xx3);
     n=size(res_spline_3a);
     for  i=1:n(2),
          res_spline_3(i) = res_spline_3a(i);
     end

xx4 =  y4(1):1:y4(length(y4));
res_spline_4=zeros(1,2048);
res_spline_4a=spline(y4,res4,xx4);
     n=size(res_spline_4a);
     for  i=1:n(2),
          res_spline_4(i) = res_spline_4a(i);
     end

fprintf(LUT,'%f\n',res_spline_1);
fprintf(LUT,'%f\n',res_spline_2);
fprintf(LUT,'%f\n',res_spline_3);
fprintf(LUT,'%f\n',res_spline_4);
fclose(LUT);

figure(1);
subplot(2,2,1)
          plot(x,transpose(y1),'+','markersize',2);
          hold on;
          plot(x,fit1(1)*x+fit1(2),'r');
          title('                                                                                                        Linearity Scan');
          ylabel('ADC counts');
          %legend('ch1',"location",'northwest');
          legend('CHAN1',[slope1 ' counts/mV'],"location", 'northwest');
          hold off;
subplot(2,2,2)
          plot(x,transpose(y2),'+','markersize',2);
          hold on;
          plot(x,fit2(1)*x+fit2(2),'r');
          %legend('ch2',"location",'northwest');
          legend('CHAN2',[slope2 ' counts/mV'],"location", 'northwest');
          hold off;
subplot(2,2,3)
          plot(x,transpose(y3),'+','markersize',2);
          hold on;
          plot(x,fit3(1)*x+fit3(2),'r');
          xlabel('DAC code');
          ylabel('ADC counts');
          %legend('ch3',"location",'northwest');
          legend('CHAN3',[slope3 ' counts/mV'],"location", 'northwest');
          hold off;
subplot(2,2,4)
          plot(x,transpose(y4),'+','markersize',2);
          hold on;
          plot(x,fit4(1)*x+fit4(2),'r');
          xlabel('DAC code');
          %legend('CHAN4',"location",'northwest');
          legend('CHAN4',[slope4 ' counts/mV'],"location", 'northwest');
          hold off;
%print('linearity_scan.jpeg');
%%print('linearity_scan.jpeg',"-S640,480");
print('linearity_scan.ps',"-dpsc2");

figure(2);
set(0,"Defaulttextfontsize",18);
subplot(2,2,1)
          plot(y1, res1,'+','markersize',2);
          hold on;
          plot(xx1, res_spline_1a, 'r');
          hold on
          legend('Fit Residuals','Cubic Spline Interpolation');
          title('CHAN1');
          %xlabel('ADC counts');
          ylabel('ADC count correction');
          hold off;
subplot(2,2,2)
          plot(y2, res2,'+','markersize',2);
          hold on;
          plot(xx2, res_spline_2a, 'r');
          hold on
          legend('Fit Residuals','Cubic Spline Interpolation');
          title('CHAN2');
          %xlabel('ADC counts');
          %ylabel('ADC count correction');
          hold off;
subplot(2,2,3)
          plot(y3, res3,'+','markersize',2);
          hold on;
          plot(xx3, res_spline_3a, 'r');
          hold on
          legend('Fit Residuals','Cubic Spline Interpolation');
          title('CHAN3');
          xlabel('ADC counts');
          ylabel('ADC count correction');
          hold off;
subplot(2,2,4)
          plot(y4, res4,'+','markersize',2);
          hold on;
          plot(xx4, res_spline_4a, 'r');
          hold on
          legend('Fit Residuals','Cubic Spline Interpolation');
          title('CHAN4');
          xlabel('ADC counts');
          %ylabel('ADC count correction');
          hold off;
%print('ADC_LUT.jpeg');
print('ADC_LUT.ps',"-dpsc2");

          %plot(y2, res2,'+','markersize',2);
          %plot(y3, res3,'+','markersize',2);
         % plot(y4, res4,'+','markersize',2);
        %  hold on;
       %
      %   plot(xx1, res_spline_1a, 'r');
       %   hold on;
       %   legend('Fit Residuals','Cubic Spline Interpolation');
       %   xlabel('ADC counts');
       %   ylabel('ADC count correction');
       %   hold off;
       %  print('SCAN_residuals_CH4.jpeg',"-S640,480");


