/*****
 
FakeStdUSBL - A libusb implementation of ezusbsys.

StdUSB libusb implementation used here uses same function interface with native stdUSB Windows WDM driver.
 
*****/

#include <string.h>
#include <stdio.h>
#include "stdUSB.h"

stdUSB::stdUSB(void) {
    stdUSB::stdHandle = INVALID_HANDLE_VALUE;
    createHandles();
}

stdUSB::~stdUSB(void) {
    if(stdHandle != INVALID_HANDLE_VALUE)
        freeHandle();
}

/**
 * Finds USBFX2 device, opens it, sets configuration, and claims interface.
 *  Using of goto is mad bad. Check out linux kernel sources.
 * @param  
 * @return bool -- SUCCEED or FAILED
 */
bool stdUSB::createHandles(void) {
    int retval;
    goto ok;

    /* on ok */
ok:
printf("in FakeStdUSB: createhandles: OK\n");
    return SUCCEED;

    /* on failure*/
fail:
printf("createhandles: FAILED\n");
    return FAILED; // Unable to open usb device. No handle.}
}


/**
 * Frees handles and resources allcated by createHandle().
 * @param  
 * @return bool -- SUCCEED or FAILED
 */
bool stdUSB::freeHandle(void) //throw(...)
{
    /* all ok */
    return SUCCEED;
}

bool stdUSB::freeHandles(void) {
    // this function exits just because there is no reason
    // to open and close handle all time
    return SUCCEED;
}

/**
 * Frees handles and resources allcated by createHandle().
 * @param data -- command, control code, value, or such to be sent to the device
 * @return bool -- SUCCEED or FAILED
 */
bool stdUSB::sendData(unsigned int data)// throw(...)
{
    /* Shifted right because send value needs to be in HEX base. char[4] ^= int (char -> 1byte, int -> 4 bytes)*/
    char buff[4];
    buff[0] = data;
    buff[1] = (data >> 8);
    buff[2] = (data >> 16);
    buff[3] = (data >> 24);

    /*  USBFX2_EP_WRITE => end point addr.
    buff => bytes to send
    USB_TOUT_MS => timeout in ms
    */
    //int retval = usb_bulk_write(stdHandle, USBFX2_EP_WRITE, buff, sizeof(buff), USB_TOUT_MS);
	int retval=4;

    printf("SendData: sent %d bytes\n", retval);

    if (retval == 4) //return value must be exact as the bytes transferred
        return SUCCEED;
    else
        return FAILED;
}

/**
 * Performs bulk transfer to IN end point. Reads data off the board.
 * @param pData -- pointer to input buffer data array
 * @param l -- size of the data buffer (think: pData[l] )
 * @param lread -- out param: number of samples read
 * @return bool -- SUCCEED or FAILED
 */
bool stdUSB::readData(unsigned short * pData, int l, int* lread)// throw(...)
{
    int buff_sz = l*sizeof(unsigned short);
    //char* buff = new char[110];
    //buff_sz = 110;
    int ichan,icell;

    /*  USBFX2_EP_READ => end point addr.
    buff => bytes to read (has to be <64 bytes)
    USB_TOUT_MS => timeout in ms
    */
    //int retval = usb_bulk_read(stdHandle, USBFX2_EP_READ, (char*)pData, buff_sz, USB_TOUT_MS);
	for (ichan=0;ichan<4;ichan++) {
		for (icell=0; icell<256; icell++) {
			pData[ichan*256+icell]=icell;
		}
	}
	int retval = 256;

    printf("readData: read %d bytes\n", retval);

    if (retval > 0) {
        //*lread = retval/buff_sz;
        *lread = (int)(retval / (unsigned long)sizeof(unsigned short));
        //*lread *= 4;
        return SUCCEED;
    } else
        printf("error code: %s\n", strerror(-1 * retval));
        *lread = retval;
        return FAILED;
}
